"Sun-1.1"

"Copyright 1992 Sun Microsystems, Inc. and Stanford University.
 See the LICENSE file for license information."

  "traits <userInterface> xBitmapMaker"
  "<prototypes userInterface> bitmapMaker"

  "Creates an X pixmap from an X bitmap file created with the
   X bitmap editor, 'bitmap'."

"
The format of X standard bitmap files is as follows:

#define foo_width 10
#define foo_height 4
#define foo_x_hot 1
#define foo_y_hot 1
static char foo_bits[] = {
   0xff, 0x03, 0x25, 0x02, 0x25, 0x02, 0xff, 0x03};

which describes the following bitmap:

   XXXXXXXXXX
   X.X..X...X
   X.X..X...X
   XXXXXXXXXX

The two hotspot #define's are optional.
This file is created by the X program 'bitmap'.
"

"
This program relies on:
    defines are written '#define' and begin the line;
    width is defined before height;
    bits are declared after width and height are defined;
    bits are defined in 2-byte hex chunks, each beginning with '0x';
    and that there isn't anything else in the file between these things
      like other declarations using braces.
"

traits userInterface _AddSlotsIfAbsent: ( | xBitmapMaker = () | )
traits xBitmapMaker _Define: ( |
    _ parent* = traits clonable.

    _ getIntegerDefineWithSuffix: suffix = (
        [ skipThroughNextDefine.
	  match: getNextWord WithSuffix: suffix ] whileFalse.
	getNextWord asInteger ).

    _ skipThroughNextDefine = (
        [ getNextWord = '#define' ] whileFalse: [ skipToNextLine ] ).

    _ whitespace = ' \t\n\r'.
	
    _ getNextWord = ( | c. w <- ''. |
        c: getNextNonWhitespace.
	[ w: w, c.
	  c: file readOneChar. ] untilTrue: [ whitespace includes: c ].
	w ).

    _ getNextNonWhitespace = ( | c |
        [ c: file readOneChar ] untilFalse: [ whitespace includes: c ].
	c ).

    _ match: word WithSuffix: suffix = ( | start |
        start: word size - suffix size.
	start < 0 ifTrue: [ ^ false ].
	(word copyFrom: start) = suffix ).

    _ skipToNextLine = ( | c |
	[ c: file readOneChar ] untilTrue: [ c = '\n' ] ).

    _ skipToBitsStart = ( | c |
        [ c: file readOneChar ] untilTrue: [ c = '{' ] ).

    _ skipToNextHexOrEnd: exit = ( | c |
	[ c: file readOneChar.
	  c = '}' ifTrue: [ exit value ].
	] untilTrue: [ c = 'x' ] ).

    _ getNextHexOrEnd: exit = ( | c. buf. |
	buf: mutableString copySize: 2.
        skipToNextHexOrEnd: exit.
	(file readInto: buf Count: 2) != 2
	  ifTrue: [ error: 'File read error' ].
	(buf at: 0), (buf at: 1) ).

    ^ parseBitmapFile: filename = ( | i <- 0. bytesAcross. |
        file: unixFile openForReading: filename.
	width:  getIntegerDefineWithSuffix: '_width'.
	height: getIntegerDefineWithSuffix: '_height'.
	bytesAcross: width /+ 8.
	data: byteVector copySize: bytesAcross * height.
	skipToBitsStart.
	[ | :exit. hex. |
	  hex: getNextHexOrEnd: exit.
	  data at: i Put: hex hexAsInteger.
	  i: i succ.
	] loopExit.
	file close.
	self ).

    ^ createBitmapForSameScreenAs: win = (
	win createBitmapForSameScreenFromData: data
	  Size: width @@ height ).

    ^ printString = 'an xBitmapMaker'.
| )

prototypes userInterface _AddSlotsIfAbsent: ( | xBitmapMaker = () | )
xBitmapMaker _Define: ( |
    _ parent* = traits xBitmapMaker.
    _ thisObjectPrints = true.

    _ file <- unixFile.
    _ height <- 0.
    _ width <- 0.
    _ data <- byteVector.
| )
