"Sun-1.5"

"Copyright 1992 Sun Microsystems, Inc. and Stanford University.
 See the LICENSE file for license information."


"Philosophy of the Xlib interface: the primitives (and their wrappers) conform
 to the standard Xlib calls as close as possible.  These Self objects put a
 very thin layer on that, mainly to make access to the Xlib functions more
 object-oriented and to simplify some Xlib functions by supplying defaults.
 Other objects in the system should be built on top of this layer to provide
 general, non Xlib specific functionality.  The idea is to keep the lowest
 layers close to Xlib so that they can be understood just by reading an Xlib
 reference manual."
 

"IMPORTANT: more slots are added to these objects in xlib.wrappers.self"


"these two slots added for 2.0.1"
traits	   _AddSlotsIfAbsent: ( | ^ windowing* = () | )
prototypes _AddSlotsIfAbsent: ( | ^ windowing* = () | )

traits     windowing _AddSlotsIfAbsent: ( | ^ xlib = () | )
prototypes windowing _AddSlotsIfAbsent: ( | ^ xlib = () | )


"--- display ---"

"Current design for X display object assumes all work is done on default screen.
 This probably should be changed in the future to allow use of any screen."

traits xlib _AddSlotsIfAbsent: ( | ^ display = () | )
traits xlib display _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().  "Slots are added here (in files generated by primitiveMaker)."

    _ whichProxy = 'xlib-display'.

    _ connections* = ( |
        ^ open = ( open: '' ).

        ^ open: dispName = (
            open: dispName IfFail: [ ^ error:
              'Could not open display \'', (xDisplayName: dispName), '\'' ] ).

        ^ open: dispName IfFail: fb = (
            (basicOpen: dispName IfFail: [ | :e | ^ fb value: e ])
	     initialize: dispName ).

    	"kjx changed _XOpenDisplayResultProxy: to
	 _XOpenDisplay_wrapxOpenDisplayNameResultProxy: for 2.0.1 VM"

    	_ basicOpen: dispName IfFail: fb = (
            dispName _XOpenDisplay_wrapxOpenDisplayNameResultProxy:
		        xlib display deadCopy
	     IfFail: [ | :e |
	       ('badTypeError' isPrefixOf: e)
	         ifFalse: [ ^ fb value: e ]
		    True: [ dispName asByteVector
			    _XOpenDisplay_wrapOpenDisplayNameResultProxy:
			      xlib display deadCopy IfFail: fb ].
	     ] ).

        _ initialize: dispName = (
            screen: defaultScreenOfDisplay.
	    gc: xlib graphicsContext defaultGCFor: self.
            name: 'X display: ',  dispName.
            file: unixFile copyFd: connectionNumber Name: name.
	    self ).

    	_ defaultGC = ( screen defaultGCOfScreen initOnDisplay: self ).

        ^ close = ( basicClose. kill. ).


    	"kjx changed _XCloseDisplayIfFail: to _XCloseDisplay_wrapxCloseIfFail:
	 for 2.0.1 VM"

    	_ basicClose = (
	    basicCloseIfFail: [ | :e |
	      ^ error: 'X display close failed: ', e ] ).
    	_ basicCloseIfFail: fb = (
    	    _XCloseDisplay_wrapxCloseIfFail: [ | :e |
	      ('deadProxyError' isPrefixOf: e)
	        ifFalse: [ ^ fb value: e ]
		   True: [ (reviveIfFail: [ | :e | fb value: e ])
			   _XCloseDisplay_wrapxCloseIfFail: fb ].
    	    ].
	    self ).
    | ).

    _ visualMatching* = ( |
	_ visualClasses* = ( |
	    staticGray  = 0.
	    grayScale   = 1.
	    staticColor = 2.
	    pseudoColor = 3.
	    trueColor   = 4.
	    directColor = 5.
	| ).

	^ matchVisualInfoToClass: c Depth: d IfFail: fb = (
    	    xMatchVisualInfoScreen: screen number
			     Depth: d
			     Class: c
			    IfFail: fb ).

    	_ isVisualClass: class Depth: depth = ( | vi |
    	    vi: matchVisualInfoToClass: class
				 Depth: depth
				IfFail: [ ^ false ].
    	    vi delete.
	    true ).

    	^ isPseudoColor8 = ( isVisualClass: pseudoColor Depth: 8 ).
	^ isGrayScale4	 = ( isVisualClass: grayScale	Depth: 4 ).
	^ isMonochrome	 = ( isVisualClass: staticGray	Depth: 1 ).
    | ).

    _ synchronization* = ( |
    	flush = ( xFlush ).
	synchronize: onoff = ( xSynchronize: onoff ).
	syncDiscardingIf: discard = ( xSyncDiscardingIf: discard ).
    | ).

    _ eventHandling* = ( |
	^ eventsPending = ( xPending ).

        ^ nextEvent = ( nonBlockingEventAction: [ blockingNextEvent ] ).

        ^ peekEvent = ( nonBlockingEventAction: [ blockingPeekEvent ] ).

        _ nonBlockingEventAction: action = (
          [ pending > 0 ifFalse: [ pending: eventsPending ].
            pending > 0 ifTrue:  [ pending: pending pred. ^ action value. ].
            file suspend.
          ] loop ).

         "blocking; should never be called except by nextEvent, above"
        _ blockingNextEvent = ( |e|
	    e: xlib events xEvent new.
            xNextEventInto: e. 
	    e refine).

         "blocking; should never be called except by peekEvent, above"
        _ blockingPeekEvent = ( |e|
	    e: xlib events xEvent new.
            xPeekEventInto: e.  
	    e refine).
    | ).

    _ atoms* = ( |
        ^ internAtom: name = ( xInternAtom: name OnlyIfExists: false ).
    | ).

    _ fonts* = ( |
    	^ loadFont: name = (
	    loadFont: name
	      IfFail: [ | :e | error: 'X font load failed: ', e ] ).

    	^ loadFont: name IfFail: fb = ( | fs |
	    fs: xLoadQueryFont: name IfFail: [ | :e | ^ fb value: e ].
	    fs display: self.
	    fs ).
    | ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ display = () | )
xlib display _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib display.

    ^ screen  <- nil "xlib screen".
    ^ name    <- 'xlib display prototype'.
    ^ gc      <- nil "xlib graphicsContext".
    _ file    <- unixFile.
    _ pending <- 0.
| )


"--- drawable ---"

traits xlib _AddSlotsIfAbsent: ( | ^ drawable = () | )
traits xlib drawable _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ polygonShapes* = ( |
        _ complex   = 0.
	_ convex    = 1.
	_ nonconvex = 2.
    | ).

    _ coordinateModes* = ( |
    	_ coordModeOrigin =   0.
	_ coordModePrevious = 1.
    | ).

    _ imageFormats* = ( |
    	_ xyBitmap = 0.
    	_ xyPixmap = 1.
	_ zPixmap  = 2.
    | ).

    _ drawing* = ( |
	^ copyArea: rect To: dest At: pt GC: gc = (
    	    display xCopyAreaSrc: self
			    Dest: dest
			      GC: gc
			    SrcX: rect left
			    SrcY: rect top
			   Width: rect width
			  Height: rect height
			   DestX: pt x
			   DestY: pt y ).

	^ copyBitmapTo: dest GC: gc 
			Width: w Height: h  
			DestX: dx DestY: dy = (
	    display copyBitmap: self 
			  Dest: dest 
			    GC: gc 
			 Width: w 
			Height: h  
			 DestX: dx 
			 DestY: dy).

	^ drawPoint: pt GC: gc = (
	    display xDrawPoint: self GC: gc X: pt x Y: pt y ).

	^ drawLine: pt1 To: pt2 GC: gc = (
    	    display xDrawLine: self GC: gc X1: pt1 x Y1: pt1 y
					   X2: pt2 x Y2: pt2 y ).

	^ drawRectangle: rect GC: gc = (
	    display xDrawRectangle: self
	    	    	    	GC: gc
				 X: rect left
				 Y: rect top
			     Width: rect width
			    Height: rect height ).

	^ fillRectangleFrom: o To: c GC: gc = (
	    display xFillRectangle: self
	    	    	    	GC: gc
				 X: o x
				 Y: o y
			     Width: c x - o x
			    Height: c y - o y ).

	^ fillRectangle: rect GC: gc = (
	    display xFillRectangle: self
	    	    	    	GC: gc
				 X: rect left
				 Y: rect top
			     Width: rect width
			    Height: rect height ).

	^ fillPolygon: ptlist GC: gc = ( | xv. yv. i <- 0. |
	    xv: vector copySize: ptlist size.
	    yv: vector copySize: ptlist size.
	    ptlist do: [ | :p |
		xv at: i Put: p x.
		yv at: i Put: p y.
		i: i + 1. ].
	    display xFillPolygon: self
    	    	    	      GC: gc
			      Xs: xv
			      Ys: yv
			   Shape: complex
			    Mode: coordModeOrigin ).

    	^ drawArcWithin: rect From: startAngle To: endAngle GC: gc = (
    	    display xDrawArc: self
			  GC: gc
			   X: rect left
			   Y: rect top
		       Width: rect width
		      Height: rect height
		  StartAngle: startAngle
		    EndAngle: endAngle ).

    	^ fillArcWithin: rect From: startAngle To: endAngle GC: gc = (
    	    display xFillArc: self
			  GC: gc
			   X: rect left
			   Y: rect top
		       Width: rect width
		      Height: rect height
		  StartAngle: startAngle
		    EndAngle: endAngle ).

	^ drawString: str At: pt GC:gc = (
	    display xDrawString: self GC: gc X: pt x Y: pt y String: str ).
    | ).

    _ querying* = ( |
    	^ pixelValueAt: pt = ( | im |
    	    im: display xGetImage: self
			      X: pt x
			      Y: pt y
			  Width: 1
			 Height: 1
		      PlaneMask: 16rffff
			 Format: zPixmap.
    	    im xGetPixelX: 0 Y: 0 ).

    	^ bestStippleSizeNear: sz = ( | w. h. |
	    w: display xQueryBestStippleWidth: self
				        Width: sz width
				       Height: sz height.
    	    h: display xQueryBestStippleHeight: self
					 Width: sz width
					Height: sz height.
    	    w @ h ).
    | ).

    ^ gc = ( display gc ).

    "for compatibility with release 2.0.1"
    ^ createBitmapForSameScreenFromData: bv Size: sz = (
    	xlib pixmap createBitmapForSameScreenAs: self FromData: bv Size: sz ).
| )


"--- window ---"

traits xlib _AddSlotsIfAbsent: ( | ^ window = () | )
traits xlib window _Define: ( |
    _ parent** = traits xlib drawable.
    _ raw* = ().

    _ whichProxy = 'xlib-window'.

    _ grabModes* = ( |
        _ grabModeSync  = 0.
        _ grabModeAsync = 1.
	_ currentTime   = 0.
    | ).

    _ creating* = ( |
	^ createOnDisplay: disp At: pos Size: sz = (
	    (((disp xCreateSimpleWindowIn: disp screen rootWindow
					X: pos x
					Y: pos y
				    Width: sz width
				   Height: sz height
			      BorderWidth: 0
				   Border: disp screen blackPixel
			       Background: disp screen whitePixel
	    ) display: disp)
	      depth: disp screen depth)
	      selfConfigurationRequest ).

	^ createDisplay: disp SubWindow:  window     X:  x
                                   Y:  y
                               Width:  width
                              Height:  height
                         BorderWidth:  bw
                              Border:  bd
                          Background:  bg
    	 = (((disp xCreateSimpleWindowIn:  window
                                   X:  x
                                   Y:  y
                               Width:  width
                              Height:  height
                         BorderWidth:  bw
                              Border:  bd
                          Background:  bg) 
				display: disp)
           			selfConfigurationRequest).

	^ name: name = ( | tp |
	    tp: xlib xTextProperty new.
	    tp xStringToTextProperty: name.
	    display xSetWMNameOfWindow: self To: tp.
	    tp delete.
	    self ).

	^ iconName: name = ( | tp |
	    tp: xlib xTextProperty new.
	    tp xStringToTextProperty: name.
	    display xSetWMIconNameOfWindow: self To: tp.
	    tp delete.
	    self ).

	_ selfConfigurationRequest = ( | sizeHints |
	    sizeHints: xlib xSizeHints new.
	    sizeHints flags: sizeHints pPosition || sizeHints pSize.
	    display xSetWMNormalHintsOfWindow: self To: sizeHints.
	    sizeHints delete.
	    self ).

	^ minSize: minSize = ( | sizeHints |
	    sizeHints: xlib xSizeHints new.
	    sizeHints flags: sizeHints pMinSize.
	    sizeHints min_width:  minSize width.
	    sizeHints min_height: minSize height.
	    display xSetWMNormalHintsOfWindow: self To: sizeHints.
	    sizeHints delete.
	    self ).

	^ iconified: iconified Input: input = (
	    iconified: iconified Icon: xlib pixmap deadCopy Input: input ).

	^ iconified: iconified Icon: icon Input: input = ( | wmHints |
	    wmHints: xlib xWMHints new.
	    wmHints initial_state:
	      iconified ifTrue: wmHints iconicState False: wmHints normalState.
	    wmHints input: input.
	    wmHints flags: (wmHints stateHint || wmHints inputHint).
	    icon isLive ifTrue: [
		wmHints icon_pixmap: icon.
		wmHints flags: wmHints flags || wmHints iconPixmapHint. ].
	    display xSetWMHintsOfWindow: self To: wmHints.
	    wmHints delete.
	    self ).

	^ catchWMDelete = (
	    xlib wmProtocols:    display internAtom: 'WM_PROTOCOLS'.
	    xlib wmDeleteWindow: display internAtom: 'WM_DELETE_WINDOW'.
	    display xSetWMProtocolOfWindow: self Protocol: xlib wmDeleteWindow.
	    self ).

	^ selectInput = (
	    display xSelectInputOfWindow: self EventMask: eventMask.  
	    self ).

    | ).

    _ manipulating* = ( |
	^ map   = ( display xMapWindow:   self ).
	^ unmap = ( display xUnmapWindow: self ).
	^ raise = ( display xRaiseWindow: self ).
	^ clear = ( display xClearWindow: self ).
	^ destroy = ( display xDestroyWindow: self ).
	^ id    = (  getWinId ).
	"HACK HACK HACK HACK ***************************:"
	^ moveResizeX: x Y: y W: w H: h = 
		(moveResizeX: x Y: y Width: w Height: h).
	^ moveResizeX: x Y: y Width: w Height: h = 
		(display xMoveResizeWindow: self 
			X: x Y: y Width: w Height: h).

    | ).

    _ cursorOps* = ( |
	^ warpPointerTo: pt = (
	    display xWarpPointerSrcWindow: xlib window nullWindow
			       DestWindow: self
				     SrcX: 0
				     SrcY: 0
				 SrcWidth: 0
				SrcHeight: 0
				    DestX: pt x
				    DestY: pt y.
	    display flush.
	    self ).

	^ defineCursor: cursor = (
	    display xDefineCursorWindow: self Cursor: cursor ).
	^ undefineCursor = ( display xUndefineCursorWindow: self ).

	^ grabPointer: cursor = (
	    display xGrabPointerWindow: self
			   OwnerEvents: true
			     EventMask: eventMask
			   PointerMode: grabModeAsync
			  KeyboardMode: grabModeAsync
			     ConfineTo: self
			        Cursor: cursor
			          Time: currentTime ).
	^ grabPointer   = ( grabPointer: xlib cursor nullCursor ).
	^ ungrabPointer = ( display xUngrabPointerTime: currentTime ).
    | ).
| )


prototypes xlib _AddSlotsIfAbsent: ( | ^ window = () | )
xlib window _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib window.

    ^ display   <- xlib display.
    ^ eventMask <- 0.
    ^ depth	<- 1.
| )


"--- cursor ---"

traits xlib _AddSlotsIfAbsent: ( | ^ cursor = () | )
traits xlib cursor _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-cursor'.

    ^ createOnDisplay: disp Shape: shape Mask: mask FG: fg BG: bg
      Hotspot: hotspot = (
        ( disp xCreatePixmapCursorSource: shape
                                    Mask: mask
                         ForegroundColor: fg 
                         BackgroundColor: bg 
				       X: hotspot x
				       Y: hotspot y
        ) display: disp ).

    ^ createInvisibleForWindow: win = (
        | bv = (byteVector copySize: 1) at: 0 Put: 0. 
          pix. color. curse. |
        pix: xlib pixmap createBitmapForSameScreenAs: win
	    	    	    	    	    FromData: bv
						Size: 1@@1.
        color: (((xlib xColor new) red: 0) green: 0) blue: 0.
        curse: createOnDisplay: win display
                         Shape: pix
                          Mask: pix
                            FG: color
                            BG: color 
                       Hotspot: 0@0.
        color delete.
        pix delete.
        curse ).

    delete = ( display xFreeCursor: self. kill. ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ cursor = () | )
xlib cursor _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib cursor.

    _ display <- xlib display.
| )

"--- xGCValues ---"

traits xlib _AddSlotsIfAbsent: ( | ^ xGCValues = () | )
traits xlib xGCValues _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xGCValues'.
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xGCValues = () | )
xlib xGCValues _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib xGCValues.
| )


"--- graphicsContext ---"

traits xlib _AddSlotsIfAbsent: ( | ^ graphicsContext = () | )
traits xlib graphicsContext _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-graphicsContext'.

    _ attributes* = ( |
    	_ gcFunction          = 1 << 0.
	_ gcPlaneMask	      = 1 << 1.
	_ gcForeground	      = 1 << 2.
	_ gcBackground	      = 1 << 3.
	_ gcLineWidth	      = 1 << 4.
	_ gcLineStyle	      = 1 << 5.
	_ gcCapStyle	      = 1 << 6.
	_ gcJoinStyle	      = 1 << 7.
	_ gcFillStyle	      = 1 << 8.
	_ gcFillRule	      = 1 << 9.
	_ gcTile	      = 1 << 10.
	_ gcStipple	      = 1 << 11.
	_ gcTileStipXOrigin   = 1 << 12.
	_ gcTileStipYOrigin   = 1 << 13.
	_ gcFont	      = 1 << 14.
	_ gcSubwindowMode     = 1 << 15.
	_ gcGraphicsExposures = 1 << 16.
	_ gcClipXOrigin	      = 1 << 17.
	_ gcClipYOrigin	      = 1 << 18.
	_ gcClipMask	      = 1 << 19.
	_ gcDashOffset	      = 1 << 20.
	_ gcDashList	      = 1 << 21.
	_ gcArcMode	      = 1 << 22.
    | ).

    _ copyFunctions* = ( |
    	^ gxClear   	 = 16r0.
	^ gxAnd		 = 16r1.
	^ gxAndReverse	 = 16r2.
	^ gxCopy	 = 16r3.
	^ gxAndInverted	 = 16r4.
	^ gxNoop	 = 16r5.
	^ gxXor		 = 16r6.
	^ gxOr		 = 16r7.
	^ gxNor		 = 16r8.
	^ gxEquiv	 = 16r9.
	^ gxInvert	 = 16ra.
	^ gxXorReverse	 = 16rb.
	^ gxCopyInverted = 16rc.
	^ gxXorInverted	 = 16rd.
	^ gxNand	 = 16re.
	^ gxSet		 = 16rf.
    | ).

    _ lineAttributes* = ( |
    	_ lineSolid      = 0.
	_ lineOnOffDash  = 1.
	_ lineDoubleDash = 2.

	_ capNotLast    = 0.
	_ capButt	= 1.
	_ capRound	= 2.
	_ capProjecting = 3.

	_ joinMiter = 0.
	_ joinRound = 1.
	_ joinBevel = 2.
    | ).

    _ fillRules* = ( |
    	_ evenOddRule = 0.
	_ windingRule = 1.
    | ).

    ^ createForSameScreenAs: db = ( | gc |
        gc: db display xCreateGCDrawable: db
			       ValueMask: 0
			          Values: xlib xGCValues new.
    	gc initOnDisplay: db display.
        gc ).

    ^ defaultGCFor: disp = (
    	disp screen defaultGCOfScreen initOnDisplay: disp ).

    _ initOnDisplay: disp = (
        display: disp.
	xgcv: xlib xGCValues new.
	self ).

    ^ delete = ( xgcv delete. display xFreeGC: self. kill. ).

    ^ foreground: fg   = ( display xSetGC: self Foreground: fg ).
    ^ background: bg   = ( display xSetGC: self Background: bg ).
    ^ function: fct    = ( display xSetGC: self Function: fct ).
    ^ plane_mask: mask = ( display xSetGC: self PlaneMask: mask ).
    ^ fillStyle: style = ( display xSetGC: self FillStyle: style ).
    ^ fillRule: rule   = ( display xSetGC: self FillRule: rule ).
    ^ stipple: stip    = ( display xSetGC: self Stipple: stip ).
    ^ font: fid	       = ( display xSetGC: self Font: fid ).
    ^ graphics_exposures: bool = (
    	display xSetGC: self GraphicsExposures: bool ).
    ^ lineWidth: w Style: l Cap: c Join: j = (
        display xSetGC: self LineAttributesWidth: w
				       LineStyle: l
				        CapStyle: c
				       JoinStyle: j ).
    ^ lineWidth: w = (
    	lineWidth: w Style: lineSolid Cap: capButt Join: joinMiter ).

    _ fillStyles = ( |
    	^ fillSolid = 0.
	^ fillTiled = 1.
	^ fillStippled = 2.
	^ fillOpaqueStippled = 3.
    | ).
    ^ fillSolid	         = ( fillStyle: fillStyles fillSolid ).
    ^ fillTiled	         = ( fillStyle: fillStyles fillTiled ).
    ^ fillStippled       = ( fillStyle: fillStyles fillStippled ).
    ^ fillOpaqueStippled = ( fillStyle: fillStyles fillOpaqueStippled ).


    _ getValues: mask = (
    	display xGetGCValuesGC: self ValueMask: mask Values: xgcv ).

    ^ foreground = ( getValues: gcForeground. xgcv foreground ).
    ^ background = ( getValues: gcBackground. xgcv background ).
    ^ function   = ( getValues: gcFunction.   xgcv function   ).
    ^ plane_mask = ( getValues: gcPlaneMask.  xgcv plane_mask ).
    ^ fill_style = ( getValues: gcFillStyle.  xgcv fill_style ).
    ^ fill_rule	 = ( getValues: gcFillRule.   xgcv fill_rule  ).
    ^ stipple	 = ( getValues: gcStipple.    xgcv stipple    ).
    ^ lineWidth  = ( getValues: gcLineWidth.  xgcv line_width ).
    ^ font       = ( getValues: gcFont.	      xgcv font	      ).
    ^ graphics_exposures = (
    	getValues: gcGraphicsExposures. xgcv graphics_exposures ).
| )


prototypes xlib _AddSlotsIfAbsent: ( | ^ graphicsContext = () | )
xlib graphicsContext _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib graphicsContext.

    _ display <- xlib display.
    _ xgcv <- xlib xGCValues.
| )


"--- pixmap ---"
 
traits xlib _AddSlotsIfAbsent: ( | ^ pixmap = () | )
traits xlib pixmap _Define: ( |
    _ parent** = traits xlib drawable.
    _ raw* = ().
    
    _ whichProxy = 'xlib-pixmap'.

    ^ createForSameScreenAs: db Size: sz Depth: dp = ( | pm |
      	pm: db display xCreatePixmap: db
	    	    	       Width: sz width
			      Height: sz height
			       Depth: dp.
	pm display: db display.
	pm size: sz.
	pm depth: dp.
	pm ).

    ^ createBitmapForSameScreenAs: db FromData: bv Size: sz = ( | pm |
        pm: db display xCreateBitmapDrawable: db
				        Data: bv
				       Width: sz width
				      Height: sz height.
        pm display: db display.
	pm size: sz.
	pm depth: 1.
	pm ).

    ^ delete = ( display xFreePixmap: self. kill. ).

    ^ drawOn: win GC: gc At: p = (
	copyBitmapTo: win GC: gc 
		Width: size width Height: size height 
		DestX: p x DestY: p y).	
| )


prototypes xlib _AddSlotsIfAbsent: ( | ^ pixmap = () | )
xlib pixmap _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib pixmap.

       display <- xlib display.
    ^_ size    <- 0 @@ 0.
    ^_ depth   <- 1.
| )


"--- visual ---"
 
traits xlib _AddSlotsIfAbsent: ( | ^ visual = () | )
traits xlib visual _Define: ( |
    parent** = traits proxy.
    raw* = ().
    whichProxy = 'xlib-visual'.

    "creating a visual by sending defaultVisual to a screen does not
     increase the C-heap"
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ visual = () | )
xlib visual _Define: proxy deadCopy _AddSlots: ( |
    parent* = traits xlib visual.
| )


"--- xVisualInfo ---"
 
traits xlib _AddSlotsIfAbsent: ( | ^ xVisualInfo = () | )
traits xlib xVisualInfo _Define: ( |
    parent** = traits proxy.
    raw* = ().
    whichProxy = 'xlib-xVisualInfo'.
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xVisualInfo = () | )
xlib xVisualInfo _Define: proxy deadCopy _AddSlots: ( |
    parent* = traits xlib xVisualInfo.
| )


"--- colormap ---"

traits xlib _AddSlotsIfAbsent: ( | ^ colormap = () | )
traits xlib colormap _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-colormap'.

    ^ createFor: win = (
        ( win display xCreateColormapWindow: win 
                                     Visual: win display screen defaultVisual
                                   Allocate: true
        ) window: win).

    ^ delete  = ( window display xFreeColormap: self. kill. ).

    ^ install = (
    	window display xSetWindow: window Colormap: self.
	self ).

    ^ installAndFlush = (
    	install.
	window display flush.
	self ).

    ^ queryOne: color = (
        window display xQueryColormap: self Color: color.
	self ).

    ^ storeOne: color = (
        window display xStoreColormap: self Color: color.
	self ).

    ^ query: colors = (
        window display xQueryColormap: self Colors: colors asVector.
        self ).

    ^ store: colors = (
        window display xStoreColormap: self Colors: colors asVector.
        self ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ colormap = () | )
xlib colormap _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib colormap.

      window.
| )


"--- xColor ---"

traits xlib _AddSlotsIfAbsent: ( | ^ xColor = () | )
traits xlib xColor _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xColor'.

    _ flagComponents* = ( |
        _ doRed   = 1 << 0.
        _ doGreen = 1 << 1.
        _ doBlue  = 1 << 2.
    | ).

    "ensures that storing & querying will affect all three RGB values"
    ^ allComponents = ( flags: doRed || doGreen || doBlue ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xColor = () | )
xlib xColor _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib xColor.
| )

traits xlib _AddSlotsIfAbsent: ( | ^ xFontStruct = () | )
traits xlib xFontStruct _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xFontStruct'.

    ^ delete = ( display xFreeFont: self. kill. ).

    ^ textWidth: str = ( xTextWidth: str ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xFontStruct = () | )
xlib xFontStruct _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib xFontStruct.

      display <- xlib display.
| )

traits xlib _AddSlotsIfAbsent: ( | ^ xCharStruct = () | )
traits xlib xCharStruct _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xFontStruct'.
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xCharStruct = () | )
xlib xCharStruct _Define: ( |
    _ parent* = traits xlib xCharStruct.
| )

traits xlib _AddSlotsIfAbsent: ( | ^ font = () | )
traits xlib font _Define: ( |
    _ parent** = traits proxy.

    _ whichProxy = 'xlib-font'.

    ^ delete = ( kill ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ font = () | )
xlib font _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib font.
| ) 


"--- screen ---"

traits xlib _AddSlotsIfAbsent: ( | ^ screen = () | )
traits xlib screen _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-screen'.

    ^ rootWindow    = ( rootWindowOfScreen ).
    ^ defaultVisual = ( defaultVisualOfScreen ).

    ^ blackPixel = ( blackPixelOfScreen ).
    ^ whitePixel = ( whitePixelOfScreen ).
    ^ width	 = ( widthOfScreen ).
    ^ height	 = ( heightOfScreen ).
    ^ size	 = ( width @@ height ).
    ^ depth	 = ( defaultDepthOfScreen ).

    ^ number	 = ( xScreenNumberOfScreen ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ screen = () | )
xlib screen _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib screen.
| )


"--- image ---"

traits xlib _AddSlotsIfAbsent: ( | ^ xImage = () | )
traits xlib xImage _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xImage'.
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xImage = () | )
xlib xImage _Define: proxy deadCopy _AddSlots: ( | 
    _ parent* = traits xlib xImage.
| )


"--- events ---"

traits xlib _AddSlotsIfAbsent: ( | ^ events = () | )
traits xlib events _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-events'.

    _ masks* = ( |
	^ noEventMask              = 0.
	^ keyPressMask             = 1 << 0.
	^ keyReleaseMask           = 1 << 1.
	^ buttonPressMask          = 1 << 2.
	^ buttonReleaseMask        = 1 << 3.
	^ enterWindowMask          = 1 << 4.
	^ leaveWindowMask          = 1 << 5.
	^ pointerMotionMask        = 1 << 6.
	^ pointerMotionHintMask    = 1 << 7.
	^ button1MotionMask        = 1 << 8.
	^ button2MotionMask        = 1 << 9.
	^ button3MotionMask        = 1 << 10.
	^ button4MotionMask        = 1 << 11.
	^ button5MotionMask        = 1 << 12.
	^ buttonMotionMask         = 1 << 13.
	^ keymapStateMask          = 1 << 14.
	^ exposureMask             = 1 << 15.
	^ visibilityChangeMask     = 1 << 16.
	^ structureNotifyMask      = 1 << 17.
	^ resizeRedirectMask       = 1 << 18.
	^ substructureNotifyMask   = 1 << 19.
	^ substructureRedirectMask = 1 << 20.
	^ focusChangeMask          = 1 << 21.
	^ propertyChangeMask       = 1 << 22.
	^ colormapChangeMask       = 1 << 23.
	^ ownerGrabButtonMask      = 1 << 24.
    | ).

    _ queueModes* = ( |
	^ queuedAlready      = 0.
	^ queuedAfterReading = 1.
	^ queuedAfterFlush   = 2.
    | ).

| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ events = () | )
xlib events _Define: proxy deadCopy _AddSlots: ( |
    _ parent** = traits xlib events.
    _ raw* = ().
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xEvent = () | )
traits xlib events xEvent _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = ( 'xlib-xEvent-', typeName ).
    ^ typeName = 'generic'.

    "compatibility with 2.0.1"
    ^ hasButtonInfo = false.

    ^ hasInputStateInfo = false.
    ^ hasLocationInfo = false.

    _ types* = ( |
	^ keyPress         = 2.
	^ keyRelease       = 3.
	^ buttonPress      = 4.
	^ buttonRelease    = 5.
	^ motionNotify     = 6.
	^ enterNotify      = 7.
	^ leaveNotify      = 8.
	^ focusIn          = 9.
	^ focusOut         = 10.
	^ keymapNotify     = 11.
	^ expose           = 12.
	^ graphicsExpose   = 13.
	^ noExpose         = 14.
	^ visibilityNotify = 15.
	^ createNotify     = 16.
	^ destroyNotify    = 17.
	^ unmapNotify      = 18.
	^ mapNotify        = 19.
	^ mapRequest       = 20.
	^ reparentNotify   = 21.
	^ configureNotify  = 22.
	^ configureRequest = 23.
	^ gravityNotify    = 24.
	^ resizeRequest    = 25.
	^ circulateNotify  = 26.
	^ circulateRequest = 27.
	^ propertyNotify   = 28.
	^ selectionClear   = 29.
	^ selectionRequest = 30.
	^ selectionNotify  = 31.
	^ colormapNotify   = 32.
	^ clientMessage    = 33.
	^ mappingNotify    = 34.
    | ).



    ^ refine = ( | t. et. |
        t: type.
	et: traits xlib events.

	motionNotify  = t ifTrue: [ ^ xmotion ].
        buttonPress   = t ifTrue: [ ^ xbutton parent: et xButtonPressedEvent  ].
        buttonRelease = t ifTrue: [ ^ xbutton parent: et xButtonReleasedEvent ].
        keyPress      = t ifTrue: [ ^ xkey      parent: et xKeyPressedEvent   ].
        keyRelease    = t ifTrue: [ ^ xkey      parent: et xKeyReleasedEvent  ].
        enterNotify   = t ifTrue: [ ^ xcrossing parent: et xEnterWindowEvent  ].
        leaveNotify   = t ifTrue: [ ^ xcrossing parent: et xLeaveWindowEvent  ].
        colormapNotify   = t ifTrue: [ ^ xcolormap       ].
        expose           = t ifTrue: [ ^ xexpose	 ].
        configureNotify  = t ifTrue: [ ^ xconfigure	 ].
        visibilityNotify = t ifTrue: [ ^ xvisibility	 ].
        mapNotify	 = t ifTrue: [ ^ xmap	    	 ].
	unmapNotify	 = t ifTrue: [ ^ xunmap		 ].
        reparentNotify	 = t ifTrue: [ ^ xreparent	 ].
        clientMessage	 = t ifTrue: [ ^ xclient	 ].
	graphicsExpose	 = t ifTrue: [ ^ xgraphicsexpose ].
	noExpose	 = t ifTrue: [ ^ xnoexpose	 ].

	xany ).

    ^ delete = ( asXEvent basicDelete. kill ).

    ^ putBackOnDisplay: d = ( | e |
        e: asXEvent.
	d xPutBackEvent: e.
	e delete.
	self ).
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xEvent = () | )
xlib events xEvent _Define: proxy  deadCopy _AddSlots: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xInputEvent = () | )
traits xlib events xInputEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    _ states* = ( |
        ^ shiftMask   = 1 << 0.
        ^ lockMask    = 1 << 1.
        ^ controlMask = 1 << 2.
        ^ mod1Mask    = 1 << 3.
        ^ mod2Mask    = 1 << 4.
        ^ mod3Mask    = 1 << 5.
        ^ mod4Mask    = 1 << 6.
        ^ mod5Mask    = 1 << 7.
        ^ button1Mask = 1 << 8.
        ^ button2Mask = 1 << 9.
        ^ button3Mask = 1 << 10.
        ^ button4Mask = 1 << 11.
        ^ button5Mask = 1 << 12.

        ^ anyButtonMask = 16r1f00.
    | ).
| )



traits xlib events _AddSlotsIfAbsent: ( | ^ xButtonEvent = () | )
traits xlib events xButtonEvent _Define: ( |
    _ parent** = traits xlib events xInputEvent.
    _ raw* = ().


    "compatibility with 2.0.1"
    ^ hasButtonInfo = true.

    ^ hasInputStateInfo = true.
    ^ hasLocationInfo = true.

    _ details* = ( |
        button1 = 1.
        button2 = 2.
        button3 = 3.
        button4 = 4.
        button5 = 5.
    | ).

    ^ buttonName = ( buttonName: button ).
      buttonName: b = (
        b = button1 ifTrue: [ ^ 'left' ].
        b = button2 ifTrue: [ ^ 'middle' ].
        b = button3 ifTrue: [ ^ 'right' ].
        error: 'unknown button'. ).

    "compatibility with 2.0.1"
    postState = ( newState ).
    postStateFrom: s Changing: b = ( newStateFrom: s Changing: b ).

    ^ newState = ( newStateFrom: state Changing: button ).
    _ newStateFrom: s Changing: b = (
        b = button1 ifTrue: [ ^ s ^^ button1Mask ].
        b = button2 ifTrue: [ ^ s ^^ button2Mask ].
        b = button3 ifTrue: [ ^ s ^^ button3Mask ].
        error: 'unknown button'. ).
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xButtonPressedEvent = () | )
traits xlib events xButtonPressedEvent _Define: ( |
    _ parent** = traits xlib events xButtonEvent.
    _ raw* = ().

    ^ typeName = 'buttonPress'.
| )

traits xlib events _AddSlotsIfAbsent: ( | ^ xButtonReleasedEvent = () | )
traits xlib events xButtonReleasedEvent _Define: ( |
    _ parent** = traits xlib events xButtonEvent.
    _ raw* = ().

    ^ typeName = 'buttonRelease'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xButtonEvent = () | )
xlib events xButtonEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent** <- traits xlib events xButtonEvent.
| )



traits xlib events _AddSlotsIfAbsent: ( | ^ xKeyEvent = () | )
traits xlib events xKeyEvent _Define: ( |
    _ parent** = traits xlib events xInputEvent.
    _ raw* = ().

    "compatibility with 2.0.1"
    ^ hasButtonInfo = true.

    ^ hasInputStateInfo = true.

    "this is specific to Sun Type 4 keyboards"     
    _ kbdModifierMasks = (((((((((dictionary copyRemoveAll
				    at: 106 Put: shiftMask)   "shift_left"
				    at: 117 Put: shiftMask)   "shift_right"
				    at: 126 Put: lockMask)    "caps_lock"
				    at: 83  Put: controlMask) "control"
				    at: 127 Put: mod1Mask)    "meta_left"
				    at: 129 Put: mod1Mask)    "meta_right"
				    at: 20  Put: mod2Mask)    "alt_graph"
				    at: 105 Put: mod3Mask)    "num_lock"
				    at: 26  Put: mod4Mask ).  "alt"

    ^ newState = ( newStateFrom: state Changing: keycode ).
    _ newStateFrom: s Changing: kc = (
    	s ^^ (kbdModifierMasks at: kc IfAbsent: 0) ).

    ^ lookupString = ( | bv = '' copySize: 5. n. |
    	n: xLookupStringReturn: bv.
	bv copySize: n ).
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xKeyPressedEvent = () | )
traits xlib events xKeyPressedEvent _Define: ( |
    _ parent** = traits xlib events xKeyEvent.
    _ raw* = ().

    ^ typeName = 'keyPress'.
| )

traits xlib events _AddSlotsIfAbsent: ( | ^ xKeyReleasedEvent = () | )
traits xlib events xKeyReleasedEvent _Define: ( |
    _ parent** = traits xlib events xKeyEvent.
    _ raw* = ().

    ^ typeName = 'keyRelease'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xKeyEvent = () | )
xlib events xKeyEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* <- traits xlib events xKeyEvent.
| )



traits xlib events _AddSlotsIfAbsent: ( | ^ xExposeEvent = () | )
traits xlib events xExposeEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'expose'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xExposeEvent = () | )
xlib events xExposeEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xExposeEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xNoExposeEvent = () | )
traits xlib events xNoExposeEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'noexpose'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xNoExposeEvent = () | )
xlib events xNoExposeEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xNoExposeEvent.
| )

traits xlib events _AddSlotsIfAbsent: ( | ^ xGraphicsExposeEvent = () | )
traits xlib events xGraphicsExposeEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'graphicsexpose'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xGraphicsExposeEvent = () | )
xlib events xGraphicsExposeEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xGraphicsExposeEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xExposeEvent = () | )
traits xlib events xExposeEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'expose'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xExposeEvent = () | )
xlib events xExposeEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xExposeEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xMotionEvent = () | )
traits xlib events xMotionEvent _Define: ( |
    _ parent** = traits xlib events xInputEvent.
    _ raw* = ().

    ^ typeName = 'motionNotify'.
    ^ hasLocationInfo = true.
    ^ newState = ( state ).
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xMotionEvent = () | )
xlib events xMotionEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xMotionEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xConfigureEvent = () | )
traits xlib events xConfigureEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ area = ((x@y)##(width@@height)).

    ^ typeName = 'configureNotify'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xConfigureEvent = () | )
xlib events xConfigureEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xConfigureEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xClientMessageEvent = () | )
traits xlib events xClientMessageEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'clientMessage'.

    ^ isDeleteWindow = ( (message_type = xlib wmProtocols) &&
			 [(atomAt: 0) = xlib wmDeleteWindow] ).
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xClientMessageEvent = () | )
xlib events xClientMessageEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xClientMessageEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xVisibilityEvent = () | )
traits xlib events xVisibilityEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'visibilityNotify'.

    _ states* = ( |
        visibilityUnobscured        = 0.
        visibilityPartiallyObscured = 1.
        visibilityFullyObscured     = 2.
    | ).
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xVisibilityEvent = () | )
xlib events xVisibilityEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xVisibilityEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xCrossingEvent = () | )
traits xlib events xCrossingEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().
| )

traits xlib events _AddSlotsIfAbsent: ( | ^ xEnterWindowEvent = () | )
traits xlib events xEnterWindowEvent _Define: ( |
    _ parent** = traits xlib events xCrossingEvent.
    _ raw* = ().

    ^ typeName = 'enterNotify'.
| )

traits xlib events _AddSlotsIfAbsent: ( | ^ xLeaveWindowEvent = () | )
traits xlib events xLeaveWindowEvent _Define: ( |
    _ parent** = traits xlib events xCrossingEvent.
    _ raw* = ().

    ^ typeName = 'leaveNotify'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xCrossingEvent = () | )
xlib events xCrossingEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* <- traits xlib events xCrossingEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xMapEvent = () | )
traits xlib events xMapEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'mapNotify'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xMapEvent = () | )
xlib events xMapEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xMapEvent.
| )

traits xlib events _AddSlotsIfAbsent: ( | ^ xUnmapEvent = () | )
traits xlib events xUnmapEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'unmapNotify'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xUnmapEvent = () | )
xlib events xUnmapEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xUnmapEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xColormapEvent = () | )
traits xlib events xColormapEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'colormapNotify'.

    _ states* = ( |
        _ colormapUninstalled = 0.
        _ colormapInstalled   = 1.
    | ).
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xColormapEvent = () | )
xlib events xColormapEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xColormapEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xReparentEvent = () | )
traits xlib events xReparentEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'reparentNotify'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xReparentEvent = () | )
xlib events xReparentEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xReparentEvent.
| )


traits xlib events _AddSlotsIfAbsent: ( | ^ xAnyEvent = () | )
traits xlib events xAnyEvent _Define: ( |
    _ parent** = traits xlib events xEvent.
    _ raw* = ().

    ^ typeName = 'otherEvent'.
| )

prototypes xlib events _AddSlotsIfAbsent: ( | ^ xAnyEvent = () | )
xlib events xAnyEvent _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib events xAnyEvent.
| )



"--- hint structures ---"

traits xlib _AddSlotsIfAbsent: ( | ^ xSizeHints = () | )
traits xlib xSizeHints _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xSizeHints'.

    _ flagBits* = ( |
        usPosition  = 1 << 0.
        usSize      = 1 << 1.
        pPosition   = 1 << 2.
        pSize       = 1 << 3.
        pMinSize    = 1 << 4.
        pMaxSize    = 1 << 5.
        pResizeInc  = 1 << 6.
        pAspect     = 1 << 7.
        pBaseSize   = 1 << 8.
        pWinGravity = 1 << 9.
    | ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xSizeHints = () | )
xlib xSizeHints _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib xSizeHints.
| )


traits xlib _AddSlotsIfAbsent: ( | ^ xWMHints = () | )
traits xlib xWMHints _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xWMHints'.

    _ flagBits* = ( |
        inputHint        = 1 << 0.
        stateHint        = 1 << 1.
        iconPixmapHint   = 1 << 2.
        iconWindowHint   = 1 << 3.
        iconPositionHint = 1 << 4.
        iconMaskHint     = 1 << 5.
        windowGroupHint  = 1 << 6.
    | ).

    _ initialStates* = ( |
        withdrawnState = 0.
        normalState    = 1.
        iconicState    = 3.
    | ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xWMHints = () | )
xlib xWMHints _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib xWMHints.
| )


traits xlib _AddSlotsIfAbsent: ( | ^ xTextProperty = () | )
traits xlib xTextProperty _Define: ( |
    _ parent** = traits proxy.
    _ raw* = ().

    _ whichProxy = 'xlib-xTextProperty'.

    ^ delete = ( | v |
    	v: value.
	freeValue: v.
	v kill.
	basicDelete. ).
| )

prototypes xlib _AddSlotsIfAbsent: ( | ^ xTextProperty = () | )
xlib xTextProperty _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib xTextProperty.
| )


"--- atom ---"

traits     xlib _AddSlotsIfAbsent: ( | ^ atom = () | )
prototypes xlib _AddSlotsIfAbsent: ( | ^ atom = () | )


traits xlib atom _Define: ( |
   _ parent** = traits proxy.
   ^ raw = ().

   _ whichProxy = 'xlib-atom'.
| )

prototypes xlib atom _Define: proxy deadCopy _AddSlots: ( |
    _ parent* = traits xlib atom.
| )        

prototypes xlib _AddSlots: ( |
    ^ wmProtocols    <- xlib atom deadCopy.
    ^ wmDeleteWindow <- xlib atom deadCopy.
| )

"must read xlib.wrappers--it adds slots into objects defined in this file"
'xlib.wrappers' _RunScript
